/***************************************************************************
 *   Copyright (C) 2003 by Roberto Natella                                 *
 *   roberto.natella@virgilio.it                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef KOMICS_H
#define KOMICS_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kpanelapplet.h>
#include <kconfig.h>
#include <kurl.h>

#include <qstring.h>
#include <qmap.h>
#include <qvaluevector.h>
#include <qtimer.h>
#include <qevent.h>

#include "komicswidget.h"
#include "komicsiface.h"

class KProcess;
namespace KIO { class Job; };
class KTempFile;
class KPopupMenu;
class KomicsPreferences;
class QDate;

typedef QMap< QString, QMap< QString,QString > > StripsMap;
typedef QMap< QString, QValueVector< QMap< QString,int > > > CriteriaMap;

static const char * __comment = "Komics: an applet for comic strips";

class Komics : public KomicsWidget, virtual public KomicsIface
{
    Q_OBJECT

    friend class KomicsPreferences;
    
public:
    Komics( QWidget * parent, KConfig * config, bool isApplet = false );
    ~Komics();

    void preferences();
    void about();

    static bool lock();
    static bool unlock();

protected:
    void showStrip(const QString &);

public slots:
    bool poll();
    void killall();
    bool download( const QString & );
    QStringList stripsList();
    QString url( const QString & );
    void shutDown();

protected slots:
    void leftclicked(QMouseEvent *);
    void rightclicked(QMouseEvent *);
    void receivedStdout( KProcess *, char *, int );
    void processExited( KProcess * );
    bool downloadStart( const KURL &, const QString &, const QString &, uint = 0 );
    void downloadData( KIO::Job *, const QByteArray & );
    void downloadResult( KIO::Job * );
    bool runCommand( const QString &, const QString & );
      
private:
    bool loadConfig();
    bool writeConfig(const StripsMap&, const CriteriaMap&);
    bool readHistory();
    bool writeHistory();
    void fillPopupMenu();
    bool getCommand(QString &);
    void subst(QString &,const QString &,const QString &) const;
    void checkIfResetIcon();
    void flushHistory();

    DCOPClient * m_client;
    KomicsWidget * m_widget;
    bool m_applet;
    int m_refresh;
    bool m_onstart;
    bool m_autodownload;
    KConfig * m_kconfig;
    KPopupMenu * m_popup;
    int m_idAbout;
    int m_idConfigure;
    int m_idQuit;
    QTimer * m_timer;
    int m_id;
    int m_itemId;
    bool m_prefopened;
    bool m_killing;
    bool m_hasplayed;
    QDate m_date;
    QString m_sound;
    StripsMap m_strips;
    CriteriaMap m_criteria;
    QValueVector< QString > m_stripsStack;
    QValueVector< QString > m_stripsServed;
    QMap< KProcess*, QMap<QString,QString> > m_processes;
    QMap< KIO::Job*, KTempFile * > m_jobshandles;
    QMap< KIO::Job*, QMap<QString,QString> > m_jobsinfo;

};

class KomicsApplet : public KPanelApplet
{

public:
    KomicsApplet(const QString& configFile = QString::null, Type t = Normal, int actions = 0,
                 QWidget *parent = 0, const char *name = 0);
    ~KomicsApplet();
    virtual int widthForHeight(int height) const;
    virtual int heightForWidth(int width) const;
    virtual void about();
    virtual void preferences();
    //virtual void help();

protected:
    void resizeEvent(QResizeEvent *);

private:
    Komics * m_komics;

};

#endif
