/***************************************************************************
 *   Copyright (C) 2003 by Roberto Natella                                 *
 *   roberto.natella@virgilio.it                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "komicswidget.h"

#include <qpainter.h>
#include <qdrawutil.h>

#include <kiconloader.h>
#include <kglobal.h>

KomicsWidget::KomicsWidget(QWidget *parent)
    : QButton(parent)
{
    setBackgroundMode(X11ParentRelative);
    setBackgroundOrigin(AncestorOrigin);
    setMouseTracking(true);

    m_pixmap = MainBarIcon( "komics1" );
    QWidget::resize( m_pixmap.size() );
    
    m_state = Waiting;
}


KomicsWidget::~KomicsWidget()
{
}

void KomicsWidget::resizeEvent(QResizeEvent *e)
{
    QString iconname;
    
    switch( m_state )
    {
        case Waiting:
            iconname = "komics1";
            break;
        case Ready:
            iconname = "komics2";
            break;
        case Loading:
            iconname = "clock";
            break;
    }    

    m_pixmap = MainBarIcon( iconname, e->size().width() );
    QWidget::resize( m_pixmap.size() );
}

void KomicsWidget::setIcon( State s )
{
    QString iconname;

    switch( s )
    {
        case Waiting:
            iconname = "komics1";
            break;
        case Ready:
            iconname = "komics2";
            break;
        case Loading:
            iconname = "clock";
            break;
    }

    m_state = s;
    m_pixmap = MainBarIcon( iconname, width() );
    repaint();
}

void KomicsWidget::paintEvent(QPaintEvent *)
{
    QPainter p(this);
    int x = (width() - m_pixmap.width()) / 2;
    int y = (height() - m_pixmap.height()) / 2;
    if ( x < 0 ) x = 0;
    if ( y < 0 ) y = 0;
    
    if(isDown() || isOn())
    {
        p.fillRect(rect(), colorGroup().brush(QColorGroup::Mid));
        qDrawWinButton(&p, 0, 0, width(), height(), colorGroup(), true);
    }

    p.drawPixmap(x , y, m_pixmap);
    p.end();
}

void KomicsWidget::mousePressEvent(QMouseEvent *e)
{
    if ( e->button() == LeftButton )
    {
        setDown(true);
        setOn(true);
        repaint();
    }
    else if ( e->button() == RightButton )
    {
        emit rightclickedSignal(e);
    }
}

void KomicsWidget::mouseReleaseEvent(QMouseEvent *e)
{
    if( e->button() == LeftButton )
    {
        setDown(false);
        setOn(false);
        repaint();
        emit leftclickedSignal(e);
    }
}

#include "komicswidget.moc"
