/*
 * Copyright (C) 2003 Roberto Natella <roberto.natella@virgilio.it>
 */

#include "pref.h"

#include <klocale.h>

#include <qframe.h>


KomicsPreferences::KomicsPreferences(Komics *k)
    : KDialogBase(Tabbed, i18n("Komics Preferences"),
                  Ok|Apply|Cancel, Ok, 0, "configDialog")
{
    // this is the base class for your preferences dialog.  it is now
    // a Treelist dialog.. but there are a number of other
    // possibilities (including Tab, Swallow, and just Plain)

    m_komics = k;
    
    QFrame *page;

    page = addVBoxPage(i18n("General"));
    m_pageOne = new KomicsPrefPageOne(page);

    page = addVBoxPage(i18n("Strips configuration"));
    m_pageTwo = new KomicsPrefPageTwo(page);
    
    connect( this, SIGNAL(okClicked()), this, SLOT(saveConfig()) );
    connect( this, SIGNAL(applyClicked()), this, SLOT(saveConfig()) );
}

void KomicsPreferences::saveConfig()
{
    bool saved = m_komics->writeConfig( m_pageTwo->m_stripsdialog->m_strips,
                                        m_pageTwo->m_stripsdialog->m_criteria );
                                      
    if( saved )
    {
        m_komics->m_strips = stripsPage()->m_stripsdialog->m_strips;
        m_komics->m_criteria = stripsPage()->m_stripsdialog->m_criteria;

        m_komics->m_stripsStack.clear();
        m_komics->m_stripsServed.clear();

        m_komics->writeHistory();     // clear history

        m_komics->m_onstart = setupPage()->m_setupdialog->m_onstart->isChecked();
        m_komics->m_autodownload = setupPage()->m_setupdialog->m_autodownload->isChecked();
        m_komics->m_refresh = setupPage()->m_setupdialog->m_refresh->value();
        m_komics->m_sound = setupPage()->m_setupdialog->m_sound->url();
        
        m_komics->m_kconfig->setGroup("General Setup");
        m_komics->m_kconfig->writeEntry( "fetch_on_start", (m_komics->m_onstart ? "true" : "false") );
        m_komics->m_kconfig->writeEntry( "autodownload", (m_komics->m_autodownload ? "true" : "false") );
        m_komics->m_kconfig->writeEntry( "refresh", QString("%1").arg(m_komics->m_refresh) );
        m_komics->m_kconfig->writeEntry( "sound", m_komics->m_sound );
        m_komics->m_kconfig->sync();

        if( m_komics->m_onstart ) m_komics->poll();

        m_komics->fillPopupMenu();
    }

}

void KomicsPreferences::setContents(StripsMap& strips, CriteriaMap& criteria)
{
    m_pageTwo->m_stripsdialog->setContents(strips,criteria);
}

KomicsPrefPageOne::KomicsPrefPageOne(QWidget *parent)
    : QVBox(parent)
{
    setSpacing(KDialog::spacingHint());
    m_setupdialog = new GeneralSetup(this);
}

KomicsPrefPageTwo::KomicsPrefPageTwo(QWidget *parent)
    : QVBox(parent)
{
    setSpacing(KDialog::spacingHint());
    m_stripsdialog = new StripDialogView(this);
}

#include "pref.moc"
