/***************************************************************************
 *   Copyright (C) 2003 by Roberto Natella                                 *
 *   roberto.natella@virgilio.it                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "stripdialogview.h"
#include "criteriadialogview.h"

#include <qcombobox.h>
#include <qdatetime.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>

#include <klistbox.h>
#include <klistview.h>
#include <klocale.h>

StripDialogView::StripDialogView(QWidget* parent, const char* name, WFlags fl)
    : StripDialog(parent, name, fl)
{
    m_criteriaList->setSelectionMode(QListView::Single);

    connect( m_addStrip, SIGNAL(clicked()), this, SLOT(addStrip()) );
    connect( m_removeStrip, SIGNAL(clicked()), this, SLOT(removeStrip()) );
    connect( m_stripsList, SIGNAL(highlighted(int)),
             this, SLOT(stripSelected(int)) );

    connect( m_stripname, SIGNAL(textChanged(const QString&)),
             this, SLOT(nameChanged(const QString&)) );
    connect( m_stripURL, SIGNAL(textChanged(const QString&)),
             this, SLOT(urlChanged(const QString&)) );
    connect( m_executable, SIGNAL(textChanged(const QString&)),
             this, SLOT(execChanged(const QString&)) );
    connect( m_saveFormat, SIGNAL(textChanged(const QString&)),
             this, SLOT(formatChanged(const QString&)) );
    connect( m_referrer, SIGNAL(textChanged(const QString&)),
             this, SLOT(referrerChanged(const QString&)) );
    connect( m_activatedTrue, SIGNAL(toggled(bool)),
             this, SLOT(activatedChanged(bool)) );

    connect( m_addCriteria, SIGNAL(clicked()), this, SLOT(addCriteria()) );
    connect( m_modifyCriteria, SIGNAL(clicked()), this, SLOT(modifyCriteria()) );
    connect( m_removeCriteria, SIGNAL(clicked()), this, SLOT(removeCriteria()) );
}

StripDialogView::~StripDialogView()
{
}

void StripDialogView::resetFields()
{
    m_criteriaList->clear();
    m_criteriaPointers.clear();
}

void StripDialogView::addStrip()
{
    QString newName = "New Strip";
    int i = 1;

    while( m_stripsList->findItem(newName,Qt::CaseSensitive) )
    {
        newName = (QString("New Strip_%1")).arg(i);
        i++;
    }

    m_stripsList->insertItem( newName );

    QListBoxItem * pointer = m_stripsList->findItem(newName, Qt::CaseSensitive);

    m_boxPointers[pointer] = newName;

    m_strips[newName]["url"] = "";
    m_strips[newName]["execute"] = "";
}

void StripDialogView::removeStrip()
{
    int index = m_stripsList->currentItem();
    QListBoxItem * item = m_stripsList->item(index);
    if( item == NULL ) return;
    m_boxPointers.erase(item);
    m_strips.erase(item->text());
    m_criteria.erase(item->text());
    m_stripsList->removeItem( index );
    resetFields();

    index = m_stripsList->currentItem();
    if( index >= 0 )
    {
        m_stripsList->setSelected( index, true );
    }
    else
    {
        m_stripname->setText( "" );
        m_stripURL->setText( "" );
        m_executable->setText( "" );
        m_saveFormat->setText( "" );
        m_referrer->setText( "" );
    }
}

void StripDialogView::setContents( StripsMap & strips, CriteriaMap & criteria )
{
    m_criteria = criteria;
    m_strips = strips;
    pushStrips();
}

void StripDialogView::pushStrips()
{
    QMap< QString, QMap<QString,QString> >::iterator it;

    for( it = m_strips.begin(); it != m_strips.end(); it++ )
    {
        m_stripsList->insertItem( it.key() );
        QListBoxItem * pointer = m_stripsList->findItem(it.key(), Qt::CaseSensitive);
        m_boxPointers[pointer] = it.key();
    }
}

void StripDialogView::stripSelected( int index )
{

    if( index < 0 ) return;

    m_criteriaList->clear();
    m_criteriaPointers.clear();

    QString strip = m_stripsList->text( index );

    if( strip.isEmpty() ) return;
    
    QValueVector< QMap< QString,int > >::iterator it;
    int i = 0;

    for( it = m_criteria[strip].begin(); it != m_criteria[strip].end(); it++ )
    {
        QString date, hour, day, inverse;

        if( (*it)["inverse"] == 1 )
        {
            inverse = "true";
        }
        else
        {
            inverse = "false";
        }

        if( (*it)["day"] < 1 )
        {
            date = "*/";
        }
        else
        {
            date = QString("%1/").arg((*it)["day"]);
        }

        if( (*it)["month"] < 1 )
        {
            date += "*";
        }
        else
        {
            date += QString("%1").arg((*it)["month"]);
        }

        if( (*it)["hour"] < 0 )
        {
            hour = "*:";
        }
        else
        {
            hour = QString("%1:").arg((*it)["hour"]);
        }

        if( (*it)["minute"] < 0 )
        {
            hour += "*";
        }
        else
        {
            hour += QString("%1").arg((*it)["minute"]);
        }

        if( (*it)["dayname"] < 1 )
        {
            day = "*";
        }
        else
        {
            day = QDate::longDayName((*it)["dayname"]);
        }

        QListViewItem * crit =
                        new QListViewItem(m_criteriaList, date, hour, day, inverse);
        m_criteriaPointers[crit] = i++;
    }

    QString url = m_strips[strip]["url"];
    QString executable = m_strips[strip]["executable"];
    QString format = m_strips[strip]["format"];
    QString referrer = m_strips[strip]["referrer"];
    bool activated = (m_strips[strip]["activated"] == "1");

    m_stripname->setText( strip );
    m_stripURL->setText( url );
    m_executable->setText( executable );
    m_saveFormat->setText( format );
    m_referrer->setText( referrer );
    m_activatedTrue->setChecked(activated);
    m_activatedFalse->setChecked(!activated);
}

void StripDialogView::nameChanged( const QString & name )
{

    if( name.isEmpty() || m_stripsList->findItem(name,Qt::CaseSensitive) )
    {
        return;
    }

    int index = m_stripsList->currentItem();
    QListBoxItem * item = m_stripsList->item( index );

    if( item == NULL ) return;

    QString orig = m_boxPointers[item];

    if( orig == name ) return;

    m_boxPointers.erase( item );

    m_strips[name] = m_strips[orig];
    m_strips.erase( orig );
    m_criteria[name] = m_criteria[orig];
    m_criteria.erase( orig );

    m_stripsList->removeItem( index );
    m_stripsList->insertItem( name );

    item = m_stripsList->findItem(name,Qt::CaseSensitive);
    m_boxPointers[item] = name;
    m_stripsList->setSelected(item,true);
}

void StripDialogView::activatedChanged( bool value )
{
    int index = m_stripsList->currentItem();
    QListBoxItem * item = m_stripsList->item( index );

    if( item == NULL ) return;

    QString strip = m_boxPointers[item];

    m_strips[strip]["activated"] = ( value ? "1" : "0" );
}

void StripDialogView::urlChanged( const QString & url )
{
    int index = m_stripsList->currentItem();
    QListBoxItem * item = m_stripsList->item( index );

    if( item == NULL ) return;

    QString strip = m_boxPointers[item];

    m_strips[strip]["url"] = url;
}

void StripDialogView::execChanged( const QString & exec )
{
    int index = m_stripsList->currentItem();
    QListBoxItem * item = m_stripsList->item( index );

    if( item == NULL ) return;

    QString strip = m_boxPointers[item];

    m_strips[strip]["executable"] = exec;
}

void StripDialogView::formatChanged( const QString & format )
{
    int index = m_stripsList->currentItem();
    QListBoxItem * item = m_stripsList->item( index );

    if( item == NULL ) return;

    QString strip = m_boxPointers[item];

    m_strips[strip]["format"] = format;
}

void StripDialogView::referrerChanged( const QString & referrer )
{
    int index = m_stripsList->currentItem();
    QListBoxItem * item = m_stripsList->item( index );

    if( item == NULL ) return;

    QString strip = m_boxPointers[item];

    m_strips[strip]["referrer"] = referrer;
}

void StripDialogView::addCriteria()
{
    QListBoxItem * stripItem = m_stripsList->item( m_stripsList->currentItem() );
    if( stripItem == NULL ) return;
    QString strip = m_boxPointers[stripItem];

    QMap< QString,int > tmp;
    tmp["day"] = -1;
    tmp["month"] = -1;
    tmp["hour"] = -1;
    tmp["minute"] = -1;
    tmp["dayname"] = -1;
    tmp["inverse"] = -1;
    m_criteria[strip].push_back(tmp);

    QListViewItem * crit = new QListViewItem(m_criteriaList,"*/*","*:*","*","false");
    m_criteriaPointers[crit] = m_criteria[strip].size()-1;
}

void StripDialogView::removeCriteria()
{
    QListViewItem * criteria = m_criteriaList->currentItem();
    if( criteria == NULL ) return;
    int index = m_criteriaList->itemIndex( criteria );

    QListBoxItem * stripItem = m_stripsList->item( m_stripsList->currentItem() );
    if( stripItem == NULL ) return;
    QString strip = m_boxPointers[stripItem];

    QValueVector< QMap< QString,int > >::iterator it = m_criteria[strip].begin();
    int i=0;
    while(i!=index) { it++; i++; }

    m_criteria[strip].erase(it);
    m_criteriaPointers.erase(criteria);
    delete criteria;
}

void StripDialogView::modifyCriteria()
{
    QListViewItem * criteria = m_criteriaList->currentItem();
    
    if( criteria == NULL )
        return;

    int index = m_criteriaPointers[criteria];

    QListBoxItem * item = m_stripsList->item( m_stripsList->currentItem() );

    if( item == NULL )
        return;

    QString strip = m_boxPointers[item];
        
    CriteriaDialogView * critdialog = new CriteriaDialogView();

    critdialog->setDateDay( m_criteria[strip][index]["day"] );
    critdialog->setDateMonth( m_criteria[strip][index]["month"] );
    critdialog->setHour( m_criteria[strip][index]["hour"] );
    critdialog->setMinute( m_criteria[strip][index]["minute"] );
    critdialog->setDayName( m_criteria[strip][index]["dayname"] );
    critdialog->setInverse( m_criteria[strip][index]["inverse"] == 1 );

    if( critdialog->exec() == QDialog::Accepted )
    {
        QString dateDay = critdialog->dateDay();
        if( dateDay == "*" )
            m_criteria[strip][index]["day"] = -1;
        else
            m_criteria[strip][index]["day"] = dateDay.toInt();

        QString dateMonth = critdialog->dateMonth();
        if( dateMonth == "*" )
            m_criteria[strip][index]["month"] = -1;
        else
            m_criteria[strip][index]["month"] = dateMonth.toInt();

        QString hour = critdialog->hour();
        if( hour == "*" )
            m_criteria[strip][index]["hour"] = -1;
        else
            m_criteria[strip][index]["hour"] = hour.toInt();

        QString minute = critdialog->minute();
        if( minute == "*" )
            m_criteria[strip][index]["minute"] = -1;
        else
            m_criteria[strip][index]["minute"] = minute.toInt();

        int dayName = critdialog->dayName();
        if( dayName == 0 )
            m_criteria[strip][index]["dayname"] = -1;
        else
            m_criteria[strip][index]["dayname"] = dayName;

        if( critdialog->inverse() )
        {
            m_criteria[strip][index]["inverse"] = 1;
            criteria->setText( 3, "true" );
        }
        else
        {
            m_criteria[strip][index]["inverse"] = 0;
            criteria->setText( 3, "false" );
        }

        criteria->setText( 0, dateDay+"/"+dateMonth );
        criteria->setText( 1, hour+":"+minute );
        criteria->setText( 2, (dayName <= 0 ? "*" : QDate::longDayName(dayName) ) );
    }

    delete critdialog;
}

#include "stripdialogview.moc"
